## Flow

You can create a simple app with the following code:

```Python 
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout, flow
class App:
    class Config(plus.AppInMemory.Config):
        is_horizontal = False
        height = "800px"
    @mark_create_layout
    def my_layout(self):
        node_ui_bs = []
        node_ui_cs = []
        for j in range(3):
            node_ui_bs.append(mui.VBox([
                mui.Typography(f"only connectable with handle id prefix PB"),
                flow.Handle("source", "right", id=f"PB-{j}")
            ]).prop(width=200,  alignItems="center", backgroundColor="white", border="1px solid black"))
        for j in range(2):
            node_ui_cs.append(mui.VBox([
                mui.Typography(f"only connectable with handle id prefix PC"),
                flow.Handle("source", "right", id=f"PC-{j}")
            ]).prop(width=200,  alignItems="center", backgroundColor="white", border="1px solid black"))
        custom_node_ui_source_a = mui.VBox([
            mui.Typography("only connectable with handle id prefix PA"),
            flow.Handle("source", "right", id="PA-0")
        ]).prop(width=200,  alignItems="center", backgroundColor="white", border="1px solid black")

        custom_node_ui_tgt_a = mui.VBox([
            mui.Typography("Node handle with PA prefix"),
            flow.Handle("target", "left", id="PA-wtfnode")
        ]).prop(width=200,  alignItems="center", backgroundColor="white", border="1px solid black")
        custom_node_ui_tgt_b = mui.VBox([
            mui.Typography("Node handle with PB prefix"),
            flow.Handle("target", "left", id="PB-wtfnode")
        ]).prop(width=200,  alignItems="center", backgroundColor="white", border="1px solid black")
        custom_node_ui_tgt_c = mui.VBox([
            mui.Typography("Node handle with PC prefix"),
            flow.Handle("target", "left", id="PC-wtfnode")
        ]).prop(width=200,  alignItems="center", backgroundColor="white", border="1px solid black")

        nodes = [
            flow.Node(id="sa", type="app", data=flow.NodeData(component=custom_node_ui_source_a), position=flow.XYPosition(0, 0)),
            flow.Node(id="sb1", type="app", data=flow.NodeData(component=node_ui_bs[0]), position=flow.XYPosition(0, 60)),
            flow.Node(id="sb2", type="app", data=flow.NodeData(component=node_ui_bs[1]), position=flow.XYPosition(0, 120)),
            flow.Node(id="sb3", type="app", data=flow.NodeData(component=node_ui_bs[2]), position=flow.XYPosition(0, 180)),
            flow.Node(id="sc1", type="app", data=flow.NodeData(component=node_ui_cs[0]), position=flow.XYPosition(0, 240)),
            flow.Node(id="sc2", type="app", data=flow.NodeData(component=node_ui_cs[1]), position=flow.XYPosition(0, 300)),

            flow.Node(id="ta", type="app", data=flow.NodeData(component=custom_node_ui_tgt_a), position=flow.XYPosition(300, 0)),
            flow.Node(id="tb", type="app", data=flow.NodeData(component=custom_node_ui_tgt_b), position=flow.XYPosition(300, 100)),
            flow.Node(id="tc", type="app", data=flow.NodeData(component=custom_node_ui_tgt_c), position=flow.XYPosition(300, 200)),
        ]
        # conn map for target handle
        valid_conn_map_target = {
            "PA": {
                "PA": 1, # target PA can connect one source PA
            },
            "PB": {
                "PA": 1, # target PB can connect one source PA
                "PB": 2, # target PB can connect two source PB
            },
            "PC": {
                "PB": 1, # target PC can connect one source PB
                "PC": 2, # target PC can connect two source PB
            },
        }
        return mui.HBox([
            flow.Flow(nodes, [], [
                flow.MiniMap(),
                flow.Controls(),
                flow.Background(),
            ]).prop(targetValidConnectMap=valid_conn_map_target),
        ]).prop(width="100%", height="100%", overflow="hidden")
```

